﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Concurrency;
using System.Diagnostics;
using System.Disposables;
using System.Diagnostics.Contracts;

namespace ReactiveXaml
{
    public class StopwatchScheduler : IScheduler, IEnableLogger
    {
        readonly TimeSpan maxAllowedTime;
        readonly IScheduler innerSched;
        readonly string errorMessage;

        public StopwatchScheduler(TimeSpan maxAllowedTime, string ErrorMessage = null, IScheduler innerSched = null)
        {
            this.maxAllowedTime = maxAllowedTime;
            this.errorMessage = ErrorMessage;
            this.innerSched = innerSched ?? RxApp.DeferredScheduler;
        }

        public DateTimeOffset Now {
            get { return innerSched.Now; }
        }

        public IDisposable Schedule(Action action, TimeSpan dueTime)
        {
            Contract.Requires(action != null);

            return innerSched.Schedule(() => {
                var sw = new Stopwatch();
                sw.Start(); action(); sw.Stop();

                if (sw.Elapsed > maxAllowedTime) {
                    string error = String.Format("{0} Time elapsed: {1}, max allowed is {2}", 
                        (errorMessage != null ? errorMessage + "\n" : ""), sw.Elapsed, maxAllowedTime);
                    this.Log().Error(error);
                    throw new Exception(error);
                }
            }, dueTime);
        }

        public IDisposable Schedule(Action action)
        {
            return Schedule(action, TimeSpan.Zero);
        }
    }
}

// vim: tw=120 ts=4 sw=4 et enc=utf8 :